/*
	FileDlg.cpp
*/
/*
	Not really copyright 2000, Andrew Martens.   All Rights somewhat Reserved.
*/

#ifndef FILEDLG_H
#include "FileDlg.h"
#endif

#ifndef _FILE_PANEL_H
#include <FilePanel.h>
#endif

const uint32 MSG_FILE_OPEN			= 'mFOP';

int return_val = 0;

int main(int, char**)
{	
	FileDlgApplication	myApplication;

	// Looks like everything was okay, let's go
	myApplication.Run();

	return(return_val);
}

FileDlgApplication::FileDlgApplication()
		  		  : BApplication("application/x-vnd.PenUp-FileDlg"), aFilePanel(0)
{ }

void FileDlgApplication::ReadyToRun()
{
		OnOpen();
}

void FileDlgApplication::ArgvReceived(int32 argc, char** argv)
{
	// initial parameter setup
	G_MODE = true;
	G_MULTI_SELECT = false;
	G_MODAL = false;
	G_FILE = true;
	G_NODE_FLAVORS = 0;
	G_SQUOTE = false;
	G_DQUOTE = false;
	G_NQUOTE = false;
	
	for (int32 i=1; i<argc; i++) 
	{
		// set modal
		if (strcmp("--modal", argv[i]) == 0)
		{
			G_MODAL = true;
		}
		else if (strcmp("--open", argv[i]) == 0)
		{
			G_MODE = true;
		}
		else if (strcmp("--save", argv[i]) == 0)
		{
			G_MODE = false;
		}
		else if (strcmp("--no-file", argv[i]) == 0)
		{
			G_FILE = false;
		}
		else if (strcmp("--dir", argv[i]) == 0)
		{
			G_NODE_FLAVORS = G_NODE_FLAVORS | B_DIRECTORY_NODE;
		}
		else if (strcmp("--multi", argv[i]) == 0)
		{
			G_MULTI_SELECT = true;
		}
		else if (strcmp("--sq", argv[i]) == 0)
		{
			G_SQUOTE = true;
		}
		else if (strcmp("--dq", argv[i]) == 0)
		{
			G_DQUOTE = true;
		}
		else if (strcmp("--nq", argv[i]) == 0)
		{
			G_NQUOTE = true;
		}
		else
		{
			printf("usage: filedlg [ --modal ] [ --help ] [ --open | --save ] [ --no-file ]\n");
			printf(" [ --dir ] [ --multi ] [ --sq | --dq | --nq ]\n\n");
			printf("--modal makes the file dialog modal\n");
			printf("--help displays this usage message\n");
			printf("--open (default) or --save is your choice of file dialog\n");
			printf("--no-file is when you don't want users to be select files\n");
			printf("--dir is when you want users too be able to select directories\n");
			printf("--multi is when you want let users select multiple files\n"); 
			printf("--sq|dq|nq is when you want all the filenames on one line:\n");
			printf("  sq is for single quotes around each filename, dq for double,\n");
			printf("  and nq for no quotes\n");
			printf("FileDlg 1.0fc3 is by Andrew Martens - amartens@interchange.ubc.ca\n");
			exit(0);
		}
		
	}
	if (G_FILE == true)
	{
		G_NODE_FLAVORS = G_NODE_FLAVORS | B_FILE_NODE;
	}
	G_NODE_FLAVORS = G_NODE_FLAVORS | B_SYMLINK_NODE;
}

void FileDlgApplication::MessageReceived(BMessage* message)
{
	entry_ref ref;
	const char* foo;
	
	switch (message->what) {
	case MSG_FILE_OPEN:
		return_val = 0;
		OnOpen();
		break;
	case B_SAVE_REQUESTED:
		return_val = 0;
		if ((message->FindRef("directory", 0, &ref) == B_OK) && 
			(message->FindString("name", &foo) == B_OK))
		{
   			Save(&ref, foo);
   		}
		break;
	case B_CANCEL:
			return_val = 1;
			PostMessage(B_QUIT_REQUESTED);
		break;
	default:
		return_val = 0;
		BApplication::MessageReceived(message);
		break;
	}
}

bool FileDlgApplication::QuitRequested()
{
	return true;
}

void FileDlgApplication::RefsReceived(BMessage* message)
{
	uint32 type;
	int32 count;
	entry_ref ref;
	
	message->GetInfo("refs", &type, &count);
	if (type != B_REF_TYPE)
		return;
	
	for (int32 i = --count; i >= 0; --i) {
   		if (message->FindRef("refs", i, &ref) == B_OK) {
   			Open(&ref);
   		}
   	}
   	
   	if (G_SQUOTE || G_DQUOTE || G_NQUOTE) { printf("\n"); }
}

void FileDlgApplication::OnOpen()
{
	if (! aFilePanel) {
		if (G_MODE == true)
		{
			aFilePanel = new BFilePanel(B_OPEN_PANEL, NULL, NULL, G_NODE_FLAVORS, G_MULTI_SELECT,
			NULL, NULL, G_MODAL, true);
		}
		else
		{
			aFilePanel = new BFilePanel(B_SAVE_PANEL, NULL, NULL, G_NODE_FLAVORS, G_MULTI_SELECT,
			NULL, NULL, G_MODAL, true);
		}
		
		//aFilePanel->Window()->SetTitle("Select File");
		//Worked when compiling under R4.5.2 ... what gives?
	}
	aFilePanel->Show();
}

void FileDlgApplication::Open(const entry_ref* ref)
{
		BEntry entry(ref, true);
		BPath path;
		char fileStr[B_FILE_NAME_LENGTH];
		entry.GetName(fileStr);
		entry.GetPath(&path);
		if (G_SQUOTE == true)
		{
			printf("'%s' ", path.Path());
		}
		else if (G_DQUOTE == true)
		{
		
			printf("\"%s\" ", path.Path());
		}
		else if (G_NQUOTE == true)
		{
		
			printf("%s ", path.Path());
		}
		else
		{
			printf("%s\n", path.Path());
		}
}

void FileDlgApplication::Save(const entry_ref* ref, const char* filename)
{
		BEntry entry(ref, true);
		BPath path;

		entry.GetPath(&path);
		if (G_SQUOTE == true)
		{
			printf("'%s/%s'\n", path.Path(), filename);
		}
		else if (G_DQUOTE == true)
		{		
			printf("\"%s/%s\"\n", path.Path(), filename);
		}
		else if (G_NQUOTE == true)
		{
		
			printf("%s/%s\n", path.Path(), filename);
		}
		else
		{
			printf("%s/%s\n", path.Path(), filename);
		}
}